<?php


namespace Drupal\ultimate_cron_views\Plugin\views\query;


use Drupal\views\Plugin\views\query\Sql;


/**
 * Class CronLogSqlQuery
 *
 * Custom views query plugin for an SQL query.
 *
 * @ViewsQuery(
 *   id = "cron_log_views_query",
 *   title = @Translation("Cron Log Views Query"),
 *   help = @Translation("Query will be generated and run using the Drupal database API.")
 * )
 */
class CronLogSqlQuery extends Sql {

  /**
   * Add a field to the query table, possibly with an alias. This will
   * automatically call ensureTable to make sure the required table
   * exists, *unless* $table is unset.
   *
   * @param $table
   *   The table this field is attached to. If NULL, it is assumed this will
   *   be a formula; otherwise, ensureTable is used to make sure the
   *   table exists.
   * @param $field
   *   The name of the field to add. This may be a real field or a formula.
   * @param $alias
   *   The alias to create. If not specified, the alias will be $table_$field
   *   unless $table is NULL. When adding formulae, it is recommended that an
   *   alias be used.
   * @param $params
   *   An array of parameters additional to the field that will control items
   *   such as aggregation functions and DISTINCT. Some values that are
   *   recognized:
   *   - function: An aggregation function to apply, such as SUM.
   *   - aggregate: Set to TRUE to indicate that this value should be
   *     aggregated in a GROUP BY.
   *
   * @return string
   *   The name that this field can be referred to as. Usually this is the alias.
   */
  public function addField($table, $field, $alias = '', $params = []) {

    if ($field == 'duration') {

      // Make sure an alias is assigned
      $alias = $alias ? $alias : $field;
      $alias = strtolower(substr($alias, 0, 60));

      // Create a field info array.
      $field_info = [
          'field' => 'ROUND(' . $table . '.end_time - ' . $table . '.start_time, 3)',
          //'table' => $table,
          'alias' => $alias,
        ] + $params;

      // Test to see if the field is actually the same or not. Due to
      // differing parameters changing the aggregation function, we need
      // to do some automatic alias collision detection:
      $base = $alias;
      $counter = 0;
      while (!empty($this->fields[$alias]) && $this->fields[$alias] != $field_info) {
        $field_info['alias'] = $alias = $base . '_' . ++$counter;
      }

      if (empty($this->fields[$alias])) {
        $this->fields[$alias] = $field_info;
      }

      // Keep track of all aliases used.
      $this->fieldAliases[$table][$field] = $alias;

      return $alias;
    }
    else {
      return parent::addField($table, $field, $alias, $params);
    }

  }

}
